@@ -56,9 +56,6 @@
 -define(TIMEOUT, 60000). %% 1 minute
 -define(NONCE_LIFETIME, 60*1000*1000). %% 1 minute (in usec)
 -define(SERVER_NAME, <<"P1 STUN library">>).
-%% RFC 6156, 9.1: "a TURN relay MUST NOT accept Teredo or 6to4 addresses".
--define(BLACKLIST, [{{8193, 0, 0, 0, 0, 0, 0, 0}, 32},   % 2001::/32 (Teredo).
-		    {{8194, 0, 0, 0, 0, 0, 0, 0}, 16}]). % 2002::/16 (6to4).
 
 -type addr() :: {inet:ip_address(), inet:port_number()}.
 
@@ -76,7 +73,7 @@
 	 max_allocs = 10             :: non_neg_integer() | infinity,
 	 shaper = none               :: stun_shaper:shaper(),
 	 max_permissions = 10        :: non_neg_integer() | infinity,
-	 blacklist = ?BLACKLIST      :: turn:blacklist(),
+	 blacklist = []              :: turn:blacklist(),
 	 auth = user                 :: anonymous | user,
 	 nonces = treap:empty()      :: treap:treap(),
 	 realm = <<"">>              :: binary(),
@@ -84,7 +81,7 @@
 	 hook_fun                    :: function() | undefined,
 	 server_name = ?SERVER_NAME  :: binary(),
 	 buf = <<>>                  :: binary(),
-	 session                     :: binary() | undefined}).
+	 session_id                  :: binary() | undefined}).
 
 %%====================================================================
 %% API
@@ -113,7 +110,8 @@ udp_recv(Sock, Addr, Port, Data, State) ->
  	{ok, Msg} ->
  	    ?LOG_DEBUG(#{verbatim => {"Received:~n~s", [stun_codec:pp(Msg)]}}),
  	    process(NewState, Msg);
- 	_ ->
+ 	Err ->
+	    ?LOG_DEBUG("Cannot parse packet: ~p", [Err]),
 	    NewState
     end.
 
@@ -158,7 +156,8 @@ handle_info({tcp, _Sock, TLSData}, StateName,
     case fast_tls:recv_data(NewState#state.sock, TLSData) of
 	{ok, Data} ->
 	    process_data(StateName, NewState, Data);
-	_Err ->
+	Err ->
+	    ?LOG_INFO("Connection failure: ~p", [Err]),
 	    {stop, normal, NewState}
     end;
 handle_info({tcp, _Sock, Data}, StateName, State) ->
@@ -172,6 +171,7 @@ handle_info({tcp_error, _Sock, _Reason}, _StateName, State) ->
     {stop, normal, State};
 handle_info({timeout, TRef, stop}, _StateName,
 	    #state{tref = TRef} = State) ->
+    ?LOG_INFO("Connection timed out"),
     {stop, normal, State};
 handle_info({timeout, _TRef, activate}, StateName, State) ->
     activate_socket(State),
@@ -286,8 +286,8 @@ process(State, _Msg) ->
 
 process(State, #stun{class = request, unsupported = [_|_] = Unsupported} = Msg,
 	Secret) ->
-    ?LOG_NOTICE("Rejecting request with unknown attribute(s): ~p",
-		[Unsupported]),
+    ?LOG_DEBUG("Rejecting request with unknown attribute(s): ~p",
+	       [Unsupported]),
     Resp = prepare_response(State, Msg),
     R = Resp#stun{class = error,
 		  'UNKNOWN-ATTRIBUTES' = Unsupported,
@@ -299,13 +299,13 @@ process(State, #stun{class = request,
     AddrPort = unmap_v4_addr(State#state.peer),
     R = case stun_codec:version(Msg) of
 	    old ->
-		?LOG_INFO("Responding to 'classic' STUN request"),
+		?LOG_DEBUG("Responding to 'classic' STUN request"),
 		Resp#stun{class = response, 'MAPPED-ADDRESS' = AddrPort};
 	    new ->
-		?LOG_INFO("Responding to STUN request"),
+		?LOG_DEBUG("Responding to STUN request"),
 		Resp#stun{class = response, 'XOR-MAPPED-ADDRESS' = AddrPort}
 	end,
-    run_hook(stun_query, State),
+    run_hook(stun_query, State, Msg),
     send(State, R, Secret);
 process(#state{use_turn = false} = State,
 	#stun{class = request} = Msg, Secret) ->
@@ -339,7 +339,7 @@ process(State, #stun{class = request,
 		    {min_port, State#state.min_port},
 		    {max_port, State#state.max_port},
 		    {hook_fun, State#state.hook_fun},
-		    {session, State#state.session} |
+		    {session_id, State#state.session_id} |
 		    if SockMod /= gen_udp ->
 			    [{owner, self()}];
 		       true ->
@@ -395,7 +395,8 @@ process_data(NextStateName, #state{buf = Buf} = State, Data) ->
 	more when size(NewBuf) < ?MAX_BUF_SIZE ->
 	    NewState = State#state{buf = NewBuf},
 	    {next_state, NextStateName, NewState};
-	_ ->
+	{error, Reason} ->
+	    ?LOG_DEBUG("Cannot parse packet: ~p", [Reason]),
 	    {stop, normal, State}
     end.
 
@@ -451,7 +452,7 @@ route_on_turn(State, Msg, Pass) ->
     end.
 
 prepare_state(Opts, Sock, Peer, SockMod) when is_list(Opts) ->
-    ID = case proplists:get_value(session, Opts) of
+    ID = case proplists:get_value(session_id, Opts) of
 	     ID0 when is_binary(ID0) ->
 		 ID0; % Stick to listener's session ID.
 	     undefined ->
@@ -570,22 +571,23 @@ prepare_state(Opts, Sock, Peer, SockMod) when is_list(Opts) ->
 		 ({certfile, _}, State) -> State;
 		 ({tls, _}, State) -> State;
 		 (tls, State) -> State;
-		 ({session, _}, State) -> State;
+		 ({session_id, _}, State) -> State;
 		 (Opt, State) ->
 		      ?LOG_ERROR("Ignoring unknown option '~p'", [Opt]),
 		      State
 	      end,
-	      #state{session = ID, peer = Peer, sock = Sock,
+	      #state{session_id = ID, peer = Peer, sock = Sock,
 		     sock_mod = SockMod, use_turn = true},
 	      Opts);
 	_ ->
-	    #state{session = ID, sock = Sock, sock_mod = SockMod, peer = Peer,
-		   hook_fun = proplists:get_value(hook_fun, Opts)}
+	    #state{session_id = ID, sock = Sock, sock_mod = SockMod,
+		   peer = Peer, hook_fun = proplists:get_value(hook_fun, Opts),
+		   auth = anonymous}
     end;
 prepare_state(State, _Sock, Peer, SockMod) ->
     ID = stun_logger:make_id(),
     stun_logger:set_metadata(stun, SockMod, ID, Peer),
-    State#state{session = ID, peer = Peer}.
+    State#state{session_id = ID, peer = Peer}.
 
 prepare_addr(IPBin) when is_binary(IPBin) ->
     prepare_addr(binary_to_list(IPBin));
@@ -698,7 +700,7 @@ maybe_starttls(_Sock, fast_tls, undefined) ->
     ?LOG_ERROR("Cannot start TLS connection: option 'certfile' is not set"),
     {error, eprotonosupport};
 maybe_starttls(Sock, fast_tls, CertFile) ->
-    fast_tls:tcp_to_tls(Sock, [{certfile, CertFile}]);
+    fast_tls:tcp_to_tls(Sock, [verify_none, {certfile, CertFile}]);
 maybe_starttls(Sock, gen_tcp, _CertFile) ->
     {ok, Sock}.
 
@@ -708,19 +710,25 @@ prepare_response(State, Msg) ->
 	  trid = Msg#stun.trid,
 	  'SOFTWARE' = State#state.server_name}.
 
-run_hook(HookName, #state{session = ID,
-			  peer = Client,
-			  sock_mod = SockMod,
-			  hook_fun = HookFun})
+run_hook(HookName,
+	 #state{session_id = ID,
+		peer = Client,
+		sock_mod = SockMod,
+		hook_fun = HookFun},
+	 #stun{'USERNAME' = User,
+	       'REALM' = Realm} = Msg)
   when is_function(HookFun) ->
     Info = #{id => ID,
+	     user => User,
+	     realm => Realm,
 	     client => Client,
-	     transport => stun_logger:encode_transport(SockMod)},
+	     transport => stun_logger:encode_transport(SockMod),
+	     version => stun_codec:version(Msg)},
     ?LOG_DEBUG("Running '~s' hook", [HookName]),
     try HookFun(HookName, Info)
     catch _:Err -> ?LOG_ERROR("Hook '~s' failed: ~p", [HookName, Err])
     end;
-run_hook(HookName, _State) ->
+run_hook(HookName, _State, _Msg) ->
     ?LOG_DEBUG("No callback function specified for '~s' hook", [HookName]),
     ok.
 
