@@ -43,6 +43,13 @@
 -define(DEFAULT_LIFETIME, 600000). %% 10 minutes
 -define(PERMISSION_LIFETIME, 300000). %% 5 minutes
 -define(CHANNEL_LIFETIME, 600000). %% 10 minutes
+-define(INITIAL_BLACKLIST, [
+	%% Could be used to circumvent blocking of loopback addresses:
+	{{0, 0, 0, 0}, 8},
+	{{0, 0, 0, 0, 0, 0, 0, 0}, 128},
+	%% RFC 6156, 9.1: "a TURN relay MUST NOT accept Teredo or 6to4 addresses".
+	{{8193, 0, 0, 0, 0, 0, 0, 0}, 32},   % 2001::/32 (Teredo).
+	{{8194, 0, 0, 0, 0, 0, 0, 0}, 16}]). % 2002::/16 (6to4).
 
 -type addr() :: {inet:ip_address(), inet:port_number()}.
 -type subnet() :: {inet:ip4_address(), 0..32} | {inet:ip6_address(), 0..128}.
@@ -51,35 +58,36 @@
 -export_type([blacklist/0]).
 
 -record(state,
-	{sock_mod = gen_udp             :: gen_udp | gen_tcp | fast_tls,
-	 sock                           :: inet:socket() | fast_tls:tls_socket() | undefined,
-	 addr = {{0,0,0,0}, 0}          :: addr(),
-	 owner = self()                 :: pid(),
-	 username = <<"">>              :: binary(),
-	 realm = <<"">>                 :: binary(),
-	 key = {<<"">>, <<"">>, <<"">>} :: {binary(), binary(), binary()},
-	 server_name = <<"">>           :: binary(),
-	 peers = #{}                    :: map(),
-	 channels = #{}                 :: map(),
-	 permissions = #{}              :: map(),
-	 max_permissions                :: non_neg_integer() | atom() | undefined,
-	 relay_ipv4_ip = {127,0,0,1}    :: inet:ip4_address(),
-	 relay_ipv6_ip                  :: inet:ip6_address() | undefined,
-	 min_port = 49152               :: non_neg_integer(),
-	 max_port = 65535               :: non_neg_integer(),
-	 relay_addr                     :: addr() | undefined,
-	 relay_sock                     :: inet:socket() | undefined,
-	 last_trid                      :: non_neg_integer() | undefined,
-	 last_pkt = <<>>                :: binary(),
-	 seq = 1                        :: non_neg_integer(),
-	 life_timer                     :: reference() | undefined,
-	 blacklist                      :: blacklist() | undefined,
-	 hook_fun                       :: function() | undefined,
-	 session                        :: binary(),
-	 rcvd_bytes = 0                 :: non_neg_integer(),
-	 rcvd_pkts = 0                  :: non_neg_integer(),
-	 sent_bytes = 0                 :: non_neg_integer(),
-	 sent_pkts = 0                  :: non_neg_integer()}).
+	{sock_mod = gen_udp                :: gen_udp | gen_tcp | fast_tls,
+	 sock                              :: inet:socket() | fast_tls:tls_socket() | undefined,
+	 addr = {{0,0,0,0}, 0}             :: addr(),
+	 owner = self()                    :: pid(),
+	 username = <<"">>                 :: binary(),
+	 realm = <<"">>                    :: binary(),
+	 key = {<<"">>, <<"">>, <<"">>}    :: {binary(), binary(), binary()},
+	 server_name = <<"">>              :: binary(),
+	 peers = #{}                       :: map(),
+	 channels = #{}                    :: map(),
+	 permissions = #{}                 :: map(),
+	 max_permissions                   :: non_neg_integer() | atom() | undefined,
+	 relay_ipv4_ip = {127,0,0,1}       :: inet:ip4_address(),
+	 relay_ipv6_ip                     :: inet:ip6_address() | undefined,
+	 min_port = 49152                  :: non_neg_integer(),
+	 max_port = 65535                  :: non_neg_integer(),
+	 relay_addr                        :: addr() | undefined,
+	 relay_sock                        :: inet:socket() | undefined,
+	 last_trid                         :: non_neg_integer() | undefined,
+	 last_pkt = <<>>                   :: binary(),
+	 seq = 1                           :: non_neg_integer(),
+	 life_timer                        :: reference() | undefined,
+	 blacklist = []                    :: blacklist(),
+	 hook_fun                          :: function() | undefined,
+	 session_id                        :: binary(),
+	 rcvd_bytes = 0                    :: non_neg_integer(),
+	 rcvd_pkts = 0                     :: non_neg_integer(),
+	 sent_bytes = 0                    :: non_neg_integer(),
+	 sent_pkts = 0                     :: non_neg_integer(),
+	 start_timestamp = get_timestamp() :: integer()}).
 
 %%====================================================================
 %% API
@@ -101,13 +109,14 @@ route(Pid, Msg) ->
 %%====================================================================
 init([Opts]) ->
     process_flag(trap_exit, true),
-    ID = proplists:get_value(session, Opts),
+    ID = proplists:get_value(session_id, Opts),
     Owner = proplists:get_value(owner, Opts),
     Username = proplists:get_value(username, Opts),
     Realm = proplists:get_value(realm, Opts),
     AddrPort = proplists:get_value(addr, Opts),
     SockMod = proplists:get_value(sock_mod, Opts),
     HookFun = proplists:get_value(hook_fun, Opts),
+    Blacklist = proplists:get_value(blacklist, Opts) ++ ?INITIAL_BLACKLIST,
     State = #state{sock_mod = SockMod,
 		   sock = proplists:get_value(sock, Opts),
 		   key = proplists:get_value(key, Opts),
@@ -116,10 +125,10 @@ init([Opts]) ->
 		   min_port = proplists:get_value(min_port, Opts),
 		   max_port = proplists:get_value(max_port, Opts),
 		   max_permissions = proplists:get_value(max_permissions, Opts),
-		   blacklist = proplists:get_value(blacklist, Opts),
 		   server_name = proplists:get_value(server_name, Opts),
 		   username = Username, realm = Realm, addr = AddrPort,
-		   session = ID, owner = Owner, hook_fun = HookFun},
+		   session_id = ID, owner = Owner, hook_fun = HookFun,
+		   blacklist = Blacklist},
     stun_logger:set_metadata(turn, SockMod, ID, AddrPort, Username),
     MaxAllocs = proplists:get_value(max_allocs, Opts),
     if is_pid(Owner) ->
@@ -255,8 +264,8 @@ active(#stun{class = request,
 		       true ->
 			    lists:min([LifeTime*1000, ?MAX_LIFETIME])
 		    end,
-	    ?LOG_INFO("Refreshing TURN allocation (lifetime: ~B seconds)",
-		      [MSecs div 1000]),
+	    ?LOG_NOTICE("Refreshing TURN allocation (lifetime: ~B seconds)",
+			[MSecs div 1000]),
 	    TRef = erlang:start_timer(MSecs, self(), stop),
 	    R = Resp#stun{class = response,
 			  'LIFETIME' = (MSecs div 1000)},
@@ -442,9 +451,10 @@ terminate(_Reason, _StateName, State) ->
        true ->
 	    ok
     end,
-    ?LOG_NOTICE("Relayed ~B KiB (in ~B B / ~B packets, out ~B B / ~B packets)",
+    ?LOG_NOTICE("Relayed ~B KiB (in ~B B / ~B packets, out ~B B / ~B packets), "
+		"duration: ~B seconds",
 		[round((RcvdBytes + SentBytes) / 1024), RcvdBytes, RcvdPkts,
-		 SentBytes, SentPkts]),
+		 SentBytes, SentPkts, get_duration(State, second)]),
     run_hook(turn_session_stop, State),
     turn_sm:del_allocation(AddrPort, Username, Realm).
 
@@ -628,6 +638,15 @@ cancel_timer(TRef) ->
             ok
     end.
 
+get_timestamp() ->
+    erlang:monotonic_time().
+
+get_duration(State, Unit) ->
+    erlang:convert_time_unit(get_duration(State), native, Unit).
+
+get_duration(#state{start_timestamp = Start}) ->
+    get_timestamp() - Start.
+
 prepare_response(State, Msg) ->
     #stun{method = Msg#stun.method,
 	  magic = Msg#stun.magic,
@@ -644,7 +663,7 @@ count_rcvd(#state{rcvd_bytes = RcvdSize,
     State#state{rcvd_bytes = RcvdSize + byte_size(Data),
 		rcvd_pkts = RcvdPkts + 1}.
 
-run_hook(HookName, #state{session = ID,
+run_hook(HookName, #state{session_id = ID,
 			  username = User,
 			  realm = Realm,
 			  addr = Client,
@@ -666,7 +685,8 @@ run_hook(HookName, #state{session = ID,
 		   Info0#{sent_bytes => SentBytes,
 			  sent_pkts => SentPkts,
 			  rcvd_bytes => RcvdBytes,
-			  rcvd_pkts => RcvdPkts}
+			  rcvd_pkts => RcvdPkts,
+			  duration => get_duration(State)}
 	   end,
     ?LOG_DEBUG("Running '~s' hook", [HookName]),
     try HookFun(HookName, Info)
